#include <bits/stdc++.h>

const int32_t MAX_N = 2e5;

const int32_t BASE[] = { 133, 147 };
const int32_t MOD[] = { (int32_t) 1e9 + 7, (int32_t) 1e9 + 9 };
const int64_t DELTA = 1e10;

int32_t basePowers[2][MAX_N + 5], invPowers[2][MAX_N + 5];

int32_t fast_pow(int32_t x, int32_t pw, int32_t mod) {
    if(pw == 1) {
        return x;
    }

    if(pw % 2 == 0) {
        int32_t aux = fast_pow(x, pw / 2, mod);
        return ((int64_t) aux * aux) % mod;
    }
    else {
        int32_t aux = fast_pow(x, pw - 1, mod);
        return ((int64_t) aux * x) % mod;
    }
}

struct Hash {
    std::vector< int32_t > h[2];

    void init(const std::string &s) {
        for(int32_t p = 0; p < 2; p++) {
            h[p].push_back(s[0] - 'a' + 1);
            for(int32_t i = 1; i < s.size(); i++) {
                h[p].push_back((h[p].back() + (int64_t) basePowers[p][i] * (s[i] - 'a' + 1)) % MOD[p]);
            }
        }
    }

    void append_letter(char c) {
        for(int32_t p = 0; p < 2; p++) {
            h[p].push_back((h[p].back() + (int64_t) basePowers[p][h[p].size()] * (c - 'a' + 1)) % MOD[p]);
        }
    }

    uint64_t get_subhash(int32_t low, int32_t high) {
        int32_t res[2];
        for(int32_t p = 0; p < 2; p++) {
            int32_t hi = h[p][high];
            int32_t lo = (low == 0 ? 0 : h[p][low - 1]);

            res[p] = hi - lo;
            if(res[p] < 0) {
                res[p] += MOD[p];
            }

            res[p] = ((int64_t) res[p] * invPowers[p][low]) % MOD[p];
        }

        return res[0] * DELTA + res[1];
    }
};

Hash h;
std::unordered_map< uint64_t, int32_t > cnt;

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(nullptr);

    for(int32_t p = 0; p < 2; p++) {
        basePowers[p][0] = 1;
        invPowers[p][0] = 1;
        for(int32_t i = 1; i <= MAX_N; i++) {
            basePowers[p][i] = ((int64_t) basePowers[p][i - 1] * BASE[p]) % MOD[p];
            invPowers[p][i] = fast_pow(basePowers[p][i], MOD[p] - 2, MOD[p]);
        }
    }

    std::string s;
    std::cin >> s;

    int32_t q;
    std::cin >> q;

    int32_t n = s.size();

    h.init(s);

    int32_t ans = 0;
    for(int32_t i = 0; i < n; i++) {
        for(int32_t j = i; j < n; j++) {
            auto res = h.get_subhash(i, j);

            cnt[res]++;
        }
    }

    ans = cnt.size();

    std::cout << ans << '\n';

    int32_t left = 0, right = n - 1;
    for(int32_t j = 0; j < q; j++) {
        int32_t type;
        std::cin >> type;

        if(type == 1) {
            char c;
            std::cin >> c;

            h.append_letter(c);

            right++;
            for(int32_t i = left; i <= right; i++) {
                auto res = h.get_subhash(i, right);

                cnt[res]++;
                if(cnt[res] == 1) {
                    ans++;
                }
            }
        }
        else {
            for(int32_t i = left; i <= right; i++) {
                auto res = h.get_subhash(left, i);

                cnt[res]--;
                if(cnt[res] == 0) {
                    ans--;
                }
            }
            left++;
        }

        std::cout << ans << '\n';
    }
}
