#include <bits/stdc++.h>

const int32_t MAX_N = 1e4;

const int32_t BASE[] = { 133, 147 };
const int32_t MOD[] = { (int32_t) 1e9 + 7, (int32_t) 1e9 + 9 };
const uint64_t DELTA = 1e9 + 7;

int32_t basePowers[2][MAX_N + 5], invPowers[2][MAX_N + 5];

int32_t fast_pow(int32_t x, int32_t pw, int32_t mod) {
    if(pw == 1) {
        return x;
    }

    if(pw % 2 == 0) {
        int32_t aux = fast_pow(x, pw / 2, mod);
        return ((int64_t) aux * aux) % mod;
    }
    else {
        int32_t aux = fast_pow(x, pw - 1, mod);
        return ((int64_t) aux * x) % mod;
    }
}

struct Hash {
    int32_t siz[2];
    int32_t h[2][MAX_N + 5];

    void init(const std::string &s) {
        for(int32_t p = 0; p < 1; p++) {
            h[p][siz[p]] = (s[0] - 'a' + 1);
            for(int32_t i = 1; i < s.size(); i++) {
                siz[p]++;
                h[p][i] = ((h[p][i - 1] + (int64_t) basePowers[p][i] * (s[i] - 'a' + 1)) % MOD[p]);
            }
        }
    }

    void append_letter(char c) {
        for(int32_t p = 0; p < 1; p++) {
            siz[p]++;
            h[p][siz[p]] = ((h[p][siz[p] - 1] + (int64_t) basePowers[p][siz[p]] * (c - 'a' + 1)) % MOD[p]);
        }
    }

    uint64_t get_subhash(int32_t low, int32_t high) {
        int32_t res[2];
        for(int32_t p = 0; p < 1; p++) {
            int32_t hi = h[p][high];
            int32_t lo = (low == 0 ? 0 : h[p][low - 1]);

            res[p] = hi - lo;
            if(res[p] < 0) {
                res[p] += MOD[p];
            }

            res[p] = ((int64_t) res[p] * invPowers[p][low]) % MOD[p];
        }

        return res[0] * DELTA + res[1];
    }
};

std::string s;
Hash h;

std::vector< int32_t > sa;
std::vector< int32_t > lcp;

bool cmp(int32_t x, int32_t y) {
    int32_t low = 1, high = std::min(h.siz[0] - x + 1, h.siz[0] - y + 1), len = 0;
    while(low <= high) {
        int32_t mid = (low + high) / 2;
        if(h.get_subhash(x, x + mid - 1) == h.get_subhash(y, y + mid - 1)) {
            len = mid;
            low = mid + 1;
        }
        else {
            high = mid - 1;
        }
    }

    if(x + len == s.size()) {
        return true;
    }
    if(y + len == s.size()) {
        return false;
    }

    return s[x + len] < s[y + len];
}

void build_suffix_array() {
    sa.resize(h.siz[0] + 1);
    for(int32_t i = 0; i <= h.siz[0]; i++) {
        sa[i] = i;
    }

    std::sort(sa.begin(), sa.end(), cmp);
}

void calc_lcp() {
    lcp.clear();
    lcp.resize(sa.size() - 1);
    for(int32_t i = 1; i < sa.size(); i++) {
        int32_t x = sa[i], y = sa[i - 1];

        int32_t low = 1, high = std::min(s.size() - x, s.size() - y) + 1, len = 0;
        while(low <= high) {
            int32_t mid = (low + high) / 2;
            if(h.get_subhash(x, x + mid - 1) == h.get_subhash(y, y + mid - 1)) {
                len = mid;
                low = mid + 1;
            }
            else {
                high = mid - 1;
            }
        }

        lcp[i - 1] = len;
    }
}

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(nullptr);

    for(int32_t p = 0; p < 1; p++) {
        basePowers[p][0] = 1;
        invPowers[p][0] = 1;
        for(int32_t i = 1; i <= MAX_N; i++) {
            basePowers[p][i] = ((int64_t) basePowers[p][i - 1] * BASE[p]) % MOD[p];
            invPowers[p][i] = fast_pow(basePowers[p][i], MOD[p] - 2, MOD[p]);
        }
    }

    std::cin >> s;

    h.init(s);

    build_suffix_array();
    calc_lcp();

    int32_t ans = s.size() - sa[0];
    for(int32_t i = 1; i < s.size(); i++) {
        ans += s.size() - sa[i] - lcp[i - 1];
    }

    std::cout << ans << " ";

    int32_t q;
    std::cin >> q;

    for(int32_t i = 0; i < q; i++) {
        int32_t type;
        std::cin >> type;

        if(type == 1) {
            char c;
            std::cin >> c;

            s += c;
            h.append_letter(c);

            sa.push_back(s.size() - 1);
            std::sort(sa.begin(), sa.end(), cmp);
        }
        else {
            int32_t currMin = sa[0], ind = 0;
            for(int32_t i = 1; i < sa.size(); i++) {
                if(sa[i] < currMin) {
                    currMin = sa[i];
                    ind = i;
                }
            }

            sa.erase(sa.begin() + ind);
        }

        if(sa.size() == 0) {
            std::cout << 0 << " ";
            continue;
        }

        calc_lcp();

        ans = s.size() - sa[0];
        for(int32_t i = 1; i < sa.size(); i++) {
            ans += s.size() - sa[i] - lcp[i - 1];
        }

        std::cout << ans << " ";
    }
}
