#include <iostream>
#include <vector>
#include <utility>
using namespace std;

const int MAXN = 75004;
const int MAXM = 150005;

int n, s, t;
int source, target;

struct edge
{
    int adj, cap, use, rev;
    edge(int adj, int cap, int use, int rev)
    {
        this -> adj = adj;
        this -> cap = cap;
        this -> use = use;
        this -> rev = rev;
    }
};

vector<edge> g[MAXN * 2];
int idx[MAXN * 2];

vector<int> uch[2], muz[2], ezi[2];
int edge_posL[MAXN], edge_posR[MAXN];
pair<int, int> edge_posM[MAXM];

int from = 0, to = 0;

void Init()
{
    ios :: sync_with_stdio(false);
    cin.tie(NULL); cout.tie(NULL);

    cin >> n >> s >> t;
    source = 0;
    target = s + t + 1;

    for (int i = 1; i <= n; ++ i)
    {
        int l, m;
        cin >> l >> m;
        g[l].emplace_back(s + m, 1, 0, g[s + m].size());
        g[s + m].emplace_back(l, 0, 0, g[l].size() - 1);

        edge_posM[i] = {l, g[l].size() - 1};
    }

    for (int i = 1; i <= s; ++ i)
    {
        g[0].emplace_back(i, 1, 0, g[i].size());
        g[i].emplace_back(0, 0, 0, g[0].size() - 1);

        edge_posL[i] = g[0].size() - 1;
    }
    for (int i = 1; i <= t; ++ i)
    {
        g[s + i].emplace_back(s + t + 1, 1, 0, g[s + t + 1].size());
        g[s + t + 1].emplace_back(s + i, 0, 0, g[s + i].size() - 1);

        edge_posR[i] = g[s + i].size() - 1;
    }
}

void toZero()
{
    from = to = 0;
    for (int i = source; i <= target; ++ i)
    {
        idx[i] = 0;
    }
    for (int i = source; i <= target; ++ i)
    {
        for (int j = 0; j < g[i].size(); ++ j)
        {
            g[i][j].use = 0;
        }
    }
}

int DFS(int u = source, int f = 1)
{
    if (u == target) return f;

    while (idx[u] < g[u].size())
    {
        int i = idx[u], ret = 0;
        if (g[u][i].use < g[u][i].cap)
        {
            g[u][i].use++;
            ret = DFS(g[u][i].adj, f);
            g[u][i].use--;
        }

        if (ret != 0)
        {
            g[u][i].use++;
            g[g[u][i].adj][g[u][i].rev].use--;
            return ret;
        }
        idx[u]++;
    }

    return 0;
}

int Prepare(int u = source, int f = 1)
{
    if (u == target) return f;

    for (int i = 0; i < g[u].size(); ++ i)
    {
        if (u > g[u][i].adj) continue;

        if (u == 0)
        {
            int ret = 0;
            if (from == 0 and to == g[u][i].adj)
            {
                ret = Prepare(g[u][i].adj, f);
                if (ret != 0)
                {
                    g[u][i].use++; // no reverse edge
                    return ret;
                }
            }
            if (from == g[u][i].adj)
            {
                ret = Prepare(g[u][i].adj, f);
                if (ret != 0)
                {
                    g[u][i].use++;
                    g[g[u][i].adj][g[u][i].rev].use--;
                    return ret;
                }
            }
            if (to == target)
            {
                ret = Prepare(g[u][i].adj, f);
                if (ret != 0)
                {
                    g[u][i].use++;
                    g[g[u][i].adj][g[u][i].rev].use--;
                    return ret;
                }
            }
        }
        if (u > 0 and u <= s)
        {
            int ret = 0;
            if (to == u)
            {
                ret = Prepare(g[u][i].adj, f);
                if (ret != 0)
                {
                    g[u][i].use++;
                    g[g[u][i].adj][g[u][i].rev].use--;
                    return ret;
                }
            }
            if (from == u and to == g[u][i].adj)
            {
                ret = Prepare(g[u][i].adj, f);
                if (ret != 0)
                {
                    g[u][i].use++; // no reverse edge
                    return ret;
                }
            }
            if (from == g[u][i].adj)
            {
                ret = Prepare(g[u][i].adj, f);
                if (ret != 0)
                {
                    g[u][i].use++;
                    g[g[u][i].adj][g[u][i].rev].use--;
                    return ret;
                }
            }
        }
        if (u > s)
        {
            int ret = Prepare(g[u][i].adj, f);
            if (ret != 0)
            {
                g[u][i].use++;
                if (from != u) g[g[u][i].adj][g[u][i].rev].use--; // no reverse edge
                return ret;
            }
        }
    }

    return 0;
}

int max_flow()
{
    if (from != 0 or to != 0)
    {
        if (Prepare() != 1) return -1;
    }

    int flow = 0;
    if (from != 0 or to != 0) flow++;
    while (true)
    {
        int f = DFS();
        if (f != 0) flow += f;
        else break;
    }
    return flow;
}

int main()
{
    Init();

    int opt = max_flow();

    for (int i = 1; i <= n; ++ i)
    {
        toZero();
        g[edge_posM[i].first][edge_posM[i].second].cap = 0;
        int new_flow = max_flow();
        if (new_flow != opt) uch[0].push_back(i);
        g[edge_posM[i].first][edge_posM[i].second].cap = 1;

        toZero();
        from = edge_posM[i].first;
        to = g[edge_posM[i].first][edge_posM[i].second].adj;
        new_flow = max_flow();
        if (new_flow != opt) uch[1].push_back(i);

      //  cout << "for " << i << ' ' << new_flow << endl;
    }

    for (int i = 1; i <= s; ++ i)
    {
        toZero();
        g[0][edge_posL[i]].cap = 0;
        int new_flow = max_flow();
        if (new_flow != opt) ezi[0].push_back(i);
        g[0][edge_posL[i]].cap = 1;

        toZero();
        from = 0;
        to = i;
        new_flow = max_flow();
        if (new_flow != opt) ezi[1].push_back(i);
    }

    for (int i = 1; i <= t; ++ i)
    {
        toZero();
        g[s + i][edge_posR[i]].cap = 0;
        int new_flow = max_flow();
        if (new_flow != opt) muz[0].push_back(i);
        g[s + i][edge_posR[i]].cap = 1;

        toZero();
        from = s + i;
        to = s + t + 1;
        new_flow = max_flow();
        if (new_flow != opt) muz[1].push_back(i);
    }

    cout << opt << ' ' << uch[0].size() << ' ' << uch[1].size() << ' ' << ezi[0].size() << ' ' << ezi[1].size() << ' ' << muz[0].size() << ' ' << muz[1].size() << endl;
    for (int x : uch[0]) cout << x << ' ';
    cout << endl;
    for (int x : uch[1]) cout << x << ' ';
    cout << endl;
    for (int x : ezi[0]) cout << x << ' ';
    cout << endl;
    for (int x : ezi[1]) cout << x << ' ';
    cout << endl;
    for (int x : muz[0]) cout << x << ' ';
    cout << endl;
    for (int x : muz[1]) cout << x << ' ';
    cout << endl;

    return 0;
}

/*
6 4 5
1 1
2 1
3 2
3 3
3 4
4 4
*/
