#include <iostream>
#include <algorithm>
#include <vector>
#include <cstdint>
#include <cstdlib>
#include <unordered_set>
#include <set>
#include <string>

using namespace std;

#define K1 17
#define K2 19
#define K3 29

#define MOD1 1000000007
#define MOD2  976953773
#define MOD3  952438367

#define MAX_MOD_POWER 100032

uint32_t k1_pow_mem[MAX_MOD_POWER], k2_pow_mem[MAX_MOD_POWER], k3_pow_mem[MAX_MOD_POWER];

void k_pow_mem_init()
{
    uint64_t curv;

    curv = 1;
    for(uint32_t i=0; i<MAX_MOD_POWER; i++)
    {
        k1_pow_mem[i]=curv;
        curv = curv * K1;
        curv = curv % MOD1;
    }

    curv = 1;
    for(uint32_t i=0; i<MAX_MOD_POWER; i++)
    {
        k2_pow_mem[i]=curv;
        curv = curv * K2;
        curv = curv % MOD2;
    }

    curv = 1;
    for(uint32_t i=0; i<MAX_MOD_POWER; i++)
    {
        k3_pow_mem[i]=curv;
        curv = curv * K3;
        curv = curv % MOD3;
    }
}

typedef pair< pair< uint32_t, uint32_t > , uint32_t > string_hash_t;

string_hash_t operator+ (string_hash_t a, string_hash_t b)
{
    string_hash_t res;

    res.first.first = a.first.first + b.first.first;
    if(res.first.first >= MOD1)
        res.first.first -= MOD1;

    res.first.second = a.first.second + b.first.second;
    if(res.first.second >= MOD2)
        res.first.second -= MOD2;

    res.second = a.second + b.second;
    if(res.second >= MOD3)
        res.second -= MOD3;

    return res;
}

string_hash_t operator- (string_hash_t a, string_hash_t b)
{
    string_hash_t res;

    int64_t tmp;

    tmp = ((int64_t)a.first.first) - b.first.first;
    if(tmp < 0)
        tmp += MOD1;
    res.first.first = tmp;

    tmp = ((int64_t)a.first.second) - b.first.second;
    if(tmp < 0)
        tmp += MOD2;
    res.first.second = tmp;

    tmp = ((int64_t)a.second) - b.second;
    if(tmp < 0)
        tmp += MOD3;
    res.second = tmp;

    return res;
}

string_hash_t mult_by_power_k (string_hash_t a, uint32_t p)
{
    string_hash_t res;
    uint64_t tmp;

    tmp = ((uint64_t)a.first.first) * k1_pow_mem[p];
    if(tmp>=MOD1)
        tmp %= MOD1;
    res.first.first = tmp;

    tmp = ((uint64_t)a.first.second) * k2_pow_mem[p];
    if(tmp>=MOD2)
        tmp %= MOD2;
    res.first.second = tmp;

    tmp = ((uint64_t)a.second) * k3_pow_mem[p];
    if(tmp>=MOD3)
        tmp %= MOD3;
    res.second = tmp;

    return res;
}

string_hash_t char_to_hash (char c, uint32_t p)
{
    string_hash_t newa;
    newa.first.first  = ((uint64_t)c*k1_pow_mem[p])%MOD1;
    newa.first.second = ((uint64_t)c*k2_pow_mem[p])%MOD2;
    newa.second       = ((uint64_t)c*k3_pow_mem[p])%MOD3;

    return newa;
}

multiset < string_hash_t > global_hash_db;
uint32_t global_hash_db_diff_ent=0;

string s;
vector<string_hash_t> s_hpref;

void make_s_hpref (vector<string_hash_t> &s_hpref, string &s)
{
    string_hash_t sc;

    sc.first.first = 0; sc.first.second = 0; sc.second = 0;
    s_hpref.resize(s.size()+1);
    s_hpref[0]=sc;

    for(uint32_t i=1; i<=s.size(); i++)
    {
        s[i-1]-='a'-1;
        string_hash_t cur = char_to_hash(s[i-1], i-1);
        sc = sc + cur;

        s_hpref[i]=sc;
    }
}

uint32_t q;

string_hash_t substr_to_all_matching_hash (uint32_t b, uint32_t e)
{
    string_hash_t res = s_hpref[e+1] - s_hpref[b];

    res = mult_by_power_k(res, MAX_MOD_POWER - 10 - b);

    return res;
}

void add_db (string_hash_t a)
{
//    if(global_hash_db.count(a)==0)
//        global_hash_db_diff_ent++;
//
//    global_hash_db.insert(a);

    auto it = global_hash_db.find(a);

    if(!( *it == a))
    {
        global_hash_db_diff_ent ++;
    }

    global_hash_db.insert(it, a);
}

void init()
{
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    k_pow_mem_init();

    cin>>s;
    make_s_hpref(s_hpref, s);

    cin>>q;

    if(q!=0)
        abort();

    s.reserve(s.size() + q);
    s_hpref.reserve(s_hpref.size()+q);

    for(uint32_t i=0; i<s.size(); i++)
    {
        for(uint32_t j=i; j<s.size(); j++)
        {
            add_db(substr_to_all_matching_hash(i, j));
        }
    }

    cout<<global_hash_db_diff_ent<<endl;

//    asm volatile ("nop");
}

int main()
{
    init();

//    string_hash_t h1 = substr_to_all_matching_hash(0, 2);
//    string_hash_t h2 = substr_to_all_matching_hash(4, 6);

//    string_hash_t h1 = substr_to_all_matching_hash(0, 0);
//    string_hash_t h2 = substr_to_all_matching_hash(2, 2);
//    string_hash_t h3 = substr_to_all_matching_hash(3, 3);

//    asm volatile ("nop");

    return 0;
}
