#include <bits/stdc++.h>

using namespace std;

const int k_N = 4e5 + 3;
const int k_Inf = 1e9;

struct MaxFlow{
    struct Edge{
        int from, to;
        int cap, flow;

        Edge(){}
        Edge(int from, int to, int cap, int flow){
            this->from = from;
            this->to = to;
            this->cap = cap;
            this->flow = flow;
        }
    };

    vector<Edge> edges;
    vector<int> adjacent[k_N];
    bool blocked_node[k_N], blocked_edge[k_N];
    bool vis[k_N];
    int curr_flow[k_N], parent[k_N];

    int n;

    int add_edge(int from, int to, int cap){
        edges.push_back({from, to, cap, 0});
        edges.push_back({to, from, 0, 0});
        adjacent[from].push_back(edges.size() - 2);
        adjacent[to].push_back(edges.size() - 1);
        return edges.size() - 2;
    }

    void set_n(int n){
        this->n = n;
    }

    void clear(){
        for(auto &edge: edges)
            edge.flow = 0;
    }

    int dfs(int start, int end){
        stack<int> st;

        for(int i = 0; i < n; ++i)
            vis[i] = false;

        st.push(start);
        vis[start] = true;
        curr_flow[start] = k_Inf;
        curr_flow[end] = 0;

        while(!st.empty()){
            int node = st.top();
            st.pop();

            for(int edge_idx: adjacent[node]){
                if(blocked_edge[edge_idx] || blocked_node[edges[edge_idx].to] || vis[edges[edge_idx].to] || edges[edge_idx].flow == edges[edge_idx].cap)
                    continue;
                int to = edges[edge_idx].to;
                curr_flow[to] = min(edges[edge_idx].cap - edges[edge_idx].flow, curr_flow[node]);
                vis[to] = true;
                st.push(to);
                parent[to] = edge_idx;
            }
        }

        if(curr_flow[end]){
            int node = end;
            while(node != start){
                edges[parent[node]].flow += curr_flow[end];
                edges[parent[node] ^ 1].flow -= curr_flow[end];
                node = edges[parent[node]].from;
            }
        }

        return curr_flow[end];
    }

    int get_max_flow(int start, int end){
        int max_flow = 0;
        while(true){
            int flow = dfs(start, end);
            if(!flow)
                return max_flow;
            max_flow += flow;
        }
    }
};

int n, s, t;
MaxFlow mf;
int edge_idx[k_N];
//vector<int> adjacent[k_N];

bool can_work[k_N];

int main(){
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> n >> s >> t;

    for(int i = 0; i < n; ++i){
        int u, v;
        cin >> u >> v;

        edge_idx[i] = mf.add_edge(u, s + v, 1);
        //adjacent[u].push_back(i);
        //adjacent[v].push_back(i);
    }

    for(int i = 1; i <= s; ++i)
        mf.add_edge(0, i, 1);
    for(int i = 1; i <= t; ++i)
        mf.add_edge(s + i, s + t + 1, 1);

    mf.set_n(s + t + 2);

    int max_flow = mf.get_max_flow(0, s + t + 1);
    cout << max_flow << " ";

    vector<int> An, As, At, Bn, Bs, Bt;
    for(int i = 0; i < n; ++i){
        mf.blocked_edge[edge_idx[i]] = true;
        mf.clear();
        if(mf.get_max_flow(0, s + t + 1) != max_flow)
            An.push_back(i + 1);

        mf.blocked_node[mf.edges[edge_idx[i]].from] = true;
        mf.blocked_node[mf.edges[edge_idx[i]].to] = true;
        mf.clear();

        if(mf.get_max_flow(0, s + t + 1) == max_flow - 1){
            can_work[mf.edges[edge_idx[i]].from] = true;
            can_work[mf.edges[edge_idx[i]].to] = true;
        }
        else
            Bn.push_back(i + 1);

        mf.blocked_node[mf.edges[edge_idx[i]].from] = false;
        mf.blocked_node[mf.edges[edge_idx[i]].to] = false;
        mf.blocked_edge[edge_idx[i]] = false;
    }

    for(int i = 1; i <= s; ++i){
        mf.blocked_node[i] = true;
        mf.clear();

        if(mf.get_max_flow(0, s + t + 1) < max_flow)
            As.push_back(i);
        if(!can_work[i])
            Bs.push_back(i);

        mf.blocked_node[i] = false;
    }

    for(int i = 1; i <= t; ++i){
        mf.blocked_node[i + s] = true;
        mf.clear();

        if(mf.get_max_flow(0, s + t + 1) < max_flow)
            At.push_back(i);
        if(!can_work[i + s])
            Bt.push_back(i);

        mf.blocked_node[i + s] = false;
    }

    cout << An.size() << " " << Bn.size() << " ";
    cout << As.size() << " " << Bs.size() << " ";
    cout << At.size() << " " << Bt.size() << "\n";

    for(int x: An)
        cout << x << " ";
    cout << "\n";
    for(int x: Bn)
        cout << x << " ";
    cout << "\n";
    for(int x: As)
        cout << x << " ";
    cout << "\n";
    for(int x: Bs)
        cout << x << " ";
    cout << "\n";
    for(int x: At)
        cout << x << " ";
    cout << "\n";
    for(int x: Bt)
        cout << x << " ";
    cout << "\n";
}
