#include <bits/stdc++.h>

using namespace std;

const int k_N = 3e5 + 3;
const int k_Log_N = 20;
const int k_A = 256;

struct SuffixArray{
    int n, log_n;
    int c[k_Log_N][k_N];
    int cnt[k_N], p[k_N], pn[k_N];

    SuffixArray(){}

    void init(const string &s){
        n = (int)s.size();
        log_n = 0;

        fill(cnt, cnt + k_A, 0);

        for(int i = 0; i < n; ++i)
            cnt[s[i]]++;
        for(int i = 1; i < k_A; ++i)
            cnt[i] += cnt[i - 1];
        for(int i = n - 1; i >= 0; --i)
            p[--cnt[s[i]]] = i;

        int classes = 1;
        c[0][p[0]] = 0;

        for(int i = 1; i < n; ++i){
            if(s[p[i]] != s[p[i - 1]])
                classes++;
            c[0][p[i]] = classes - 1;
        }

        for(int h = 1, step = 1; h < n; h <<= 1, ++step){
            log_n = step;

            for(int i = 0; i < n; ++i){
                pn[i] = p[i] - h;
                if(pn[i] < 0)
                    pn[i] += n;
            }

            fill(cnt, cnt + classes, 0);

            for(int i = 0; i < n; ++i)
                cnt[c[step - 1][pn[i]]]++;
            for(int i = 1; i < n; ++i)
                cnt[i] += cnt[i - 1];
            for(int i = n - 1; i >= 0; --i)
                p[--cnt[c[step - 1][pn[i]]]] = pn[i];

            classes = 1;
            c[step][p[0]] = 0;

            for(int i = 1; i < n; ++i){
                pair<int, int> current{c[step - 1][p[i]], c[step - 1][(p[i] + h) % n]};
                pair<int, int> previous{c[step - 1][p[i - 1]], c[step - 1][(p[i - 1] + h) % n]};
                if(previous != current)
                    classes++;
                c[step][p[i]] = classes - 1;
            }
        }
    }

    int& operator[](int index){
        return p[index];
    }

    int lcp(int i, int j){
        i = p[i];
        j = p[j];

        int answer = 0;
        for(int k = log_n; k >= 0; --k){
            if(c[k][i] == c[k][j]){
                answer += (1 << k);
                i += (1 << k);
                j += (1 << k);
            }
        }

        return answer;
    }
};

struct SegmentTree{
    int mn[4 * k_N], mx[4 * k_N];

    SegmentTree(){}

    void init(int index, int l, int r){
        if(l == r){
            mn[index] = k_N;
            mx[index] = -1;
            return;
        }

        int mid = (l + r) >> 1;
        init(2 * index + 1, l, mid);
        init(2 * index + 2, mid + 1, r);

        mn[index] = k_N;
        mx[index] = -1;
    }

    void update(int index, int l, int r, int s, bool value){
        if(s < l || r < s)
            return;
        if(l == r){
            if(!value){
                mn[index] = k_N;
                mx[index] = -1;
            }
            else{
                mn[index] = l;
                mx[index] = l;
            }
            return;
        }

        int mid = (l + r) >> 1;
        update(2 * index + 1, l, mid, s, value);
        update(2 * index + 2, mid + 1, r, s, value);

        mn[index] = min(mn[2 * index + 1], mn[2 * index + 2]);
        mx[index] = max(mx[2 * index + 1], mx[2 * index + 2]);
    }

    int query_left(int index, int l, int r, int s){
        if(l > s)
            return -1;
        if(r <= s)
            return mx[index];

        int mid = (l + r) >> 1;
        int lvalue = query_left(2 * index + 1, l, mid, s);
        int rvalue = query_left(2 * index + 2, mid + 1, r, s);

        return max(lvalue, rvalue);
    }

    int query_right(int index, int l, int r, int s){
        if(r < s)
            return k_N;
        if(l >= s)
            return mn[index];

        int mid = (l + r) >> 1;
        int lvalue = query_right(2 * index + 1, l, mid, s);
        int rvalue = query_right(2 * index + 2, mid + 1, r, s);

        return min(lvalue, rvalue);
    }
};

struct FenwickTree{
    int arr[k_N];

    FenwickTree(){}

    void update(int index, int diff){
        for(++index; index < k_N; index += (index & -index))
            arr[index] += diff;
    }

    int query(int index){
        int answer = 0;
        for(++index; index >= 1; index -= (index & -index))
            answer += arr[index];
        return answer;
    }

    void update_r(int l, int r, int diff){
        update(l, diff);
        update(r + 1, -diff);
    }

    int query_r(int index){
        return query(index);
    }
};

string s;
SuffixArray suffix_array;

int q;
pair<short, char> queries[k_N];
SegmentTree st;
FenwickTree fenwick;

int log_n;

int get_diff(int index, bool rem, int right_end){
    int l = st.query_left(0, 0, s.size() - 1, suffix_array.c[log_n][index] - 1);
    int r = st.query_right(0, 0, s.size() - 1, suffix_array.c[log_n][index] + 1);

    int l_index;
    if(l != -1)
        l_index = suffix_array[l];
    int r_index;
    if(r != k_N)
        r_index = suffix_array[r];

    //cout << l << " " << r << "\n";

    int answer = 0;
    if(l != -1 && r != k_N){
        int t = max(l_index, r_index) + suffix_array.lcp(l, r) - 1;
        //cout << t << " t1" << endl;
        fenwick.update_r(0, t, rem ? 1 : -1);
        answer -= suffix_array.lcp(l, r);
        if(t > right_end)
            answer += t - right_end;
    }
    if(l != -1){
        int t = max(l_index, index) + suffix_array.lcp(l, suffix_array.c[log_n][index]) - 1;
        //cout << t << " t2" << endl;
        fenwick.update_r(0, t, rem ? -1 : 1);
        answer += suffix_array.lcp(l, suffix_array.c[log_n][index]);
        if(t > right_end)
            answer -= t - right_end;
    }
    if(r != k_N){
        int t = max(r_index, index) + suffix_array.lcp(r, suffix_array.c[log_n][index]) - 1;
        //cout << t << " t3" << endl;
        fenwick.update_r(0, t, rem ? -1 : 1);
        answer += suffix_array.lcp(suffix_array.c[log_n][index], r);
        if(t > right_end)
            answer -= t - right_end;
    }

    if(rem){
        answer = -answer;
        //cout << answer << " answer\n";
    }
    return answer;
}

int main(){
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> s;
    long long l = 0, r = s.size() - 1;
    //s += "#";
    //suffix_array.init(s);

    cin >> q;
    for(int i = 0; i < q; ++i){
        cin >> queries[i].first;
        if(queries[i].first == 1){
            cin >> queries[i].second;
            s += queries[i].second;
        }
    }

    s += "#";
    suffix_array.init(s);

    st.init(0, 0, s.size() - 1);

    log_n = suffix_array.log_n;
    long long sum = 0;
    for(int i = 0; i <= r; ++i){
        sum += fenwick.query_r(i);
        sum += get_diff(i, false, i);
        st.update(0, 0, s.size() - 1, suffix_array.c[log_n][i], true);
        //cout << sum << " sum" << endl;
    }

    cout << (r - l + 1) * (r - l + 2) / 2 - sum << " ";

    for(int i = 0; i < q; ++i){
        if(queries[i].first == 1){
            r++;
            sum += fenwick.query_r(r);
            sum += get_diff(r, false, r);

            st.update(0, 0, s.size() - 1, suffix_array.c[log_n][r], true);
        }
        else{
                while(true);
            sum += get_diff(l, true, r);
            st.update(0, 0, s.size() - 1, suffix_array.c[log_n][l], false);
            l++;
        }

        cout << (r - l + 1) * (r - l + 2) / 2 - sum << " ";
    }
    cout << "\n";
}
/*
abbababa
5
1 a
1 b
2
1 a
2
*/
