#include <iostream>
#include <string>
#include <map>
#include <iomanip>

using namespace std;

const int MOD1 = 29;

const int MaxPows = 100000;

string a;
int starting_ind;

struct big_int
{
private:
    long long a, b;
    static const long long max_d = 1e14;
public:
    big_int (int x = 0)
    {
        a = 0;
        b = x;
    }
    big_int operator + (const big_int& A) const
    {
        big_int res;
        res.b = (b + A.b) % max_d;
        res.a = (b + A.b) / max_d + (a + A.a);
        return res;
    }
    big_int operator - (const big_int& A) const
    {
        big_int res;
        res.b = ((b - A.b < 0) ? (b - A.b + max_d) : (b - A.b));
        res.a = ((b - A.b < 0) ? -1 : 0) + (a + A.a);
        return res;
    }
    big_int operator * (unsigned int x) const
    {
        big_int res;
        res.b = (b * x) % max_d;
        res.a = (b * x) / max_d + (a * x);
        return res;
    }
    big_int operator += (const big_int& A)
    {
        big_int res = (*this) + A;
        this->b = res.b;
        this->a = res.a;
        return res;
    }
    big_int operator -= (const big_int& A)
    {
        big_int res = (*this) - A;
        this->b = res.b;
        this->a = res.a;
        return res;
    }
    big_int operator *= (unsigned int x)
    {
        big_int res = (*this) * x;
        this->b = res.b;
        this->a = res.a;
        return res;
    }
    bool operator < (const big_int& A) const
    {
        return a < A.a or (a == A.a and b < A.b);
    }
    bool operator == (const big_int& A) const
    {
        return (a == A.a and b == A.b);
    }
    bool operator > (const big_int& A) const
    {
        return a > A.a or (a == A.a and b > A.b);
    }
    friend ostream& operator << (ostream&, const big_int&);
} pows1 [MaxPows];
ostream& operator << (ostream& x, const big_int& a)
{
    /*
    if (a.a == 0)
        x << a.b;
    else*/
        x << a.a << setw(15) << a.b;
    return x;
}

struct rolling_hash
{
private:
    big_int code1;
    int size;
public:
    rolling_hash ()
    {
        code1 = 0;
        size = 0;
    }
    void gen (string a)
    {
        size = a.size () - starting_ind;
        for (int i = starting_ind ; i < a.size () ; i ++)
            code1 = code1 * MOD1 + (a [i] - 'a' + 1);
    }
    void roll (char a, char b)
    {
        //cout << "rolling from " << a << " to " << b << endl;
        //cout << pows1 [size - 1] << endl;
        code1 -= pows1 [size - 1] * (a - 'a' + 1);
        //cout << "subed " << a << endl;
        code1 *= MOD1;
        //cout << "adding free space" << endl;
        code1 += b - 'a' + 1;
        //cout << "setting " << b << endl;
    }
    void add (char a)
    {
        size ++;
        code1 *= MOD1;
        code1 += a - 'a' + 1;
    }
    void remove_first (char x)
    {
        code1 -= pows1 [size - 1] * (x - 'a' + 1);
        size --;
    }
    bool operator < (const rolling_hash& hash) const
    {
        return code1 < hash.code1;
    }
    bool operator == (const rolling_hash& hash) const
    {
        return code1 == hash.code1;
    }
    bool operator > (const rolling_hash& hash) const
    {
        return code1 > hash.code1;
    }
    friend ostream& operator << (ostream&, const rolling_hash&);
};

ostream& operator << (ostream& out, const rolling_hash& A)
{
    out << A.code1;
    return out;
}

map < rolling_hash, int > substrings;

void type1 (char);
void type2 ();
void pre ();
void print (bool);

int main ()
{
    cin.tie (nullptr);
    ios::sync_with_stdio (false);
    {
        big_int _ = 1;
        for (int i = 0 ; i < MaxPows ; i ++)
        {
            pows1 [i] = _;
            _ *= MOD1;
        }
    }
    cin >> a;
    int n;
    cin >> n;
    pre ();
    print (false);
    for (int i = 0 ; i < n ; i ++)
    {
        int type;
        cin >> type;
        if (type == 1)
        {
            char A;
            cin >> A;
            //cout << "########### Type 1: " << A << "\n";
            type1 (A);
        }
        else
        {
            //cout << "########### Type 2:\n";
            type2 ();
        }
        print (i == n - 1);
    }
    return 0;
}

void type1 (char x)
{
    a += x;
    //pre ();
    rolling_hash hash;
    //cout << "String is: " << a << endl;
    hash.gen (a);
    //cout << "Just added " << hash << "(string: " << a.substr (starting_ind) << ") (current number: " << substrings [hash] << ")" << endl;
    //cout << "Gen: " << hash << "\n";
    substrings [hash] ++;
    for (int size = a.size () - starting_ind ; size >= 2 ; size --)
    {
        hash.remove_first (a [a.size () - size]);
        //cout << "Just added " << hash << "(string: " << a.substr (a.size () - size + 1) << ") (current number: " << substrings [hash] << ")" << endl;
        //cout << "Remove first (" << a [a.size () - size] << "): " << hash << "\n";
        substrings [hash] ++;
    }
}

void type2 ()
{
    rolling_hash hash;
    hash.add (a [starting_ind]);
    {
        substrings [hash] --;
        if (substrings [hash] == 0)
            substrings.erase (substrings.find (hash));
    }
    for (int i = starting_ind + 1 ; i < a.size () ; i ++)
    {
        hash.add (a [i]);
        {
            substrings [hash] --;
            if (substrings [hash] == 0)
                substrings.erase (substrings.find (hash));
        }
    }
    starting_ind ++;
    //pre ();
}

void pre ()
{
    rolling_hash hash;
    for (int size = starting_ind + 1 ; size <= a.size () ; size ++)
    {
        //cout << "size = " << size << endl;
        hash.add (a [size - 1]);
        //cout << "\tstart point: " << 0 << endl;
        //cout << "\t\t" << hash << endl;
        substrings [hash] ++;
        rolling_hash local = hash;
        for (int j = size; j < a.size () ; j ++)
        {
            //cout << "\tstart point: " << j - size + 1 << endl;
            local.roll (a [j - size], a [j]);
            //cout << "\t\t" << local << endl;
            substrings [local] ++;
        }
    }
}

void print (bool X)
{
    //for (auto& x : substrings)
    //{
    //    cout << x.first << " x " << x.second << "\n";
    //}
    cout << substrings.size ();//<< "\n---------------------\n";
    if (X)
        cout << endl;
    else
        cout << " ";
}
