#include<iostream>
#include<iomanip>
#include<cmath>
using namespace std;
#define ld long double

int n;

const int maxn = 200;
ld x[maxn], y[maxn], r[maxn];

void printm(ld m[][4])
{
    cout << "============\n";
    for(int i = 0; i < 3; i ++)
    {
        for(int j = 0; j < 4; j ++)
        {
            cout << m[i][j] << " ";
        }
        cout << endl;
    }
    cout << endl;
}


void fnd_tangent(int ind[], ld &_x, ld &_y, ld &_r)
{
    ld m[3][4];
    for(int i = 0; i < 3; i ++)
    {
        m[i][0] = -2*x[ind[i]];
        m[i][1] = -2*y[ind[i]];
        m[i][2] = -2*r[ind[i]];
        m[i][3] = x[ind[i]] * x[ind[i]] + y[ind[i]] * y[ind[i]] - r[ind[i]] * r[ind[i]];
    }

    /// substract 0
    for(int i = 1; i < 3; i ++)
    {
        for(int j = 0; j < 4; j ++)
        {
            m[i][j] -= m[0][j];
        }
    }

    ld _d = m[2][0] / m[1][0];
    ld a1 = m[2][1] - _d * m[1][1]; /// y
    ld b1 = m[2][2] - _d * m[1][2]; /// r
    ld c1 = m[2][3] - _d * m[1][3]; /// c


    /// y = (-c1/a1 - b1/a1 * r)

    ld a2 = m[1][0]; /// x
    ld b2 = m[1][2] + m[1][1] * (-b1/a1); /// r
    ld c2 = m[1][3] + m[1][1] * (-c1/a1); /// c



    /// x = (-c2 / a2 - b2/a2*r)

    c1 /= -a1;
    b1 /= -a1;
    c2 /= -a2;
    b2 /= -a2;

    //cout << a1 << " " << b1 << " " << c1 << endl;
    //cout << a2 << " " << b2 << " " << c2 << endl;


    for(int i = 1; i < 3; i ++)
    {
        for(int j = 0; j < 4; j ++)
        {
            m[i][j] += m[0][j];
        }
    }
    /// m[0][0] * x + m[0][1] * y + m[0][2] * r + m[0][3] = r*r-x*x-y*y
    /// m[0][0] * (c2  + b2 *r) + m[0][1] * (c1 + b1 * r) + m[0][2] * r + m[0][3] = r^2 - b2*b2*r^2 -2*c2*b2*r - c2*c2  - b1*b1*r^2 -2*c1*b1*r - c1*c1;
    m[0][3] += m[0][0] * c2;
    m[0][3] += m[0][1] * c1;
    m[0][2] += m[0][0] * b2;
    m[0][2] += m[0][1] * b1;

    m[0][3] += c2 * c2;
    m[0][3] += c1 * c1;
    m[0][2] += 2 * b2 * c2;
    m[0][2] += 2 * b1 * c1;

    ld a = b2*b2+b1*b1-1;
    ld b = m[0][2];
    ld c = m[0][3];


    /// a * r^2 + b * r + c = 0

    ld D = sqrt(b * b - 4 * a * c);

    ld r1 = (-b - D) / (2 * a);
    ld r2 = (-b + D) / (2 * a);

    if(r1 > 0)  _r = r1;
    if(r2 > 0)  _r = r2;

    _x = c2 + b2 * _r;
    _y = c1 + b1 * _r;

    //cout << _x << " " << _y << " " << _r << endl;

}

const ld eps = 1e-10;

bool check(ld _x, ld _y, ld _r)
{
    if(_x < 0 || _x > 1000 || _y < 0 || _y > 1000)
    {
            //cout << "bust boundary" << endl;
        return false;
    }

    for(int i = 0; i < n; i ++)
    {
        if((x[i] - _x) * (x[i] - _x) + (y[i] - _y) * (y[i] - _y) < (r[i] + _r) * (r[i] + _r) - eps)
        {
            //cout << "bust " << i << endl;
            return false;
        }
    }
    //cout << "ok\n";
    return true;
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(0);


    cout << setprecision(11) << fixed;

    cin >> n;
    for(int i = 0; i < n; i ++)
    {
        cin >> x[i] >> y[i] >> r[i];
    }

    ld curr_best = 0;
    for(int i = 0; i < n; i ++)
    {
        for(int j = i + 1; j < n; j ++)
        {
            for(int k = j + 1; k < n; k ++)
            {
                int temp[] = {i, j, k};
                ld _x, _y, _r;
                fnd_tangent(temp, _x, _y, _r);
                if(_r > curr_best && check(_x, _y, _r))
                {
                    curr_best = _r;
                }
            }
        }
    }
    cout << curr_best;
}
/**
3
42.1337 13.666 191.17
987.987 666.666 321.321
133.331 666.666666666 123.321
*/


/**
7
42.1337 13.666 191.17
543.21 765.432 55.555
987.987 666.666 321.321
0 1000 100
666.666 66.66 16.6
133.331 666.666666666 123.321
888.8 88.8 8.8
*/
