#include<iostream>
#include<iomanip>
#include<cmath>
using namespace std;
#define ld long double

int n;

const int maxn = 200;
ld x[maxn], y[maxn], r[maxn];

void fnd_tangent(int ind[], ld &_x, ld &_y, ld &_r)
{
    ld m[3][4];
    for(int i = 0; i < 3; i ++)
    {
        m[i][0] = -2*x[ind[i]];
        m[i][1] = -2*y[ind[i]];
        m[i][2] = -2*r[ind[i]];
        m[i][3] = x[ind[i]] * x[ind[i]] + y[ind[i]] * y[ind[i]] - r[ind[i]] * r[ind[i]];
    }

    /// substract 0
    for(int i = 1; i < 3; i ++)
    {
        for(int j = 0; j < 4; j ++)
        {
            m[i][j] -= m[0][j];
        }
    }

    ld _d = m[2][0] / m[1][0];
    ld a1 = m[2][1] - _d * m[1][1]; /// y
    ld b1 = m[2][2] - _d * m[1][2]; /// r
    ld c1 = m[2][3] - _d * m[1][3]; /// c


    /// y = (-c1/a1 - b1/a1 * r)

    ld a2 = m[1][0]; /// x
    ld b2 = m[1][2] + m[1][1] * (-b1/a1); /// r
    ld c2 = m[1][3] + m[1][1] * (-c1/a1); /// c



    /// x = (-c2 / a2 - b2/a2*r)

    c1 /= -a1;
    b1 /= -a1;
    c2 /= -a2;
    b2 /= -a2;

    //cout << a1 << " " << b1 << " " << c1 << endl;
    //cout << a2 << " " << b2 << " " << c2 << endl;


    for(int i = 1; i < 3; i ++)
    {
        for(int j = 0; j < 4; j ++)
        {
            m[i][j] += m[0][j];
        }
    }
    /// m[0][0] * x + m[0][1] * y + m[0][2] * r + m[0][3] = r*r-x*x-y*y
    /// m[0][0] * (c2  + b2 *r) + m[0][1] * (c1 + b1 * r) + m[0][2] * r + m[0][3] = r^2 - b2*b2*r^2 -2*c2*b2*r - c2*c2  - b1*b1*r^2 -2*c1*b1*r - c1*c1;
    m[0][3] += m[0][0] * c2;
    m[0][3] += m[0][1] * c1;
    m[0][2] += m[0][0] * b2;
    m[0][2] += m[0][1] * b1;

    m[0][3] += c2 * c2;
    m[0][3] += c1 * c1;
    m[0][2] += 2 * b2 * c2;
    m[0][2] += 2 * b1 * c1;

    ld a = b2*b2+b1*b1-1;
    ld b = m[0][2];
    ld c = m[0][3];


    /// a * r^2 + b * r + c = 0

    if(a != 0)
    {
        ld D = sqrt(b * b - 4 * a * c);

        ld r1 = (-b - D) / (2 * a);
        ld r2 = (-b + D) / (2 * a);

        if(r1 > 0)  _r = r1;
        if(r2 > 0)  _r = r2;
    } else {
        _r = -c / b;
    }

    _x = c2 + b2 * _r;
    _y = c1 + b1 * _r;

    //cout << _x << " " << _y << " " << _r << endl;
}

void fnd_online(ld x1, ld y1, ld r1, ld x2, ld y2, ld r2, ld &_x, ld &_y, ld &_r)
{
    ld a1 = 2 * y1 - 2 * y2;
    ld b1 = 2 * r1 - 2 * r2;
    ld c1 = x2 * x2 - x1 * x1 + y2 * y2 - y1 * y1 - r2 * r2 + r1 * r1;
    b1 /= -a1;
    c1 /= -a1;
    ///y = b1 * r + c1

    ld a = b1 * b1 - 1;
    ld b = 2 * b1 * c1 - 2 * b1 * y1 -2 * r1;
    ld c = x1 * x1 + c1 * c1 + y1 * y1 - 2 * c1 * y1 - r1 * r1;

    if(a != 0)
    {
        ld D = sqrt(b * b - 4 * a * c);

        ld _r1 = (-b - D) / (2 * a);
        ld _r2 = (-b + D) / (2 * a);

        if(_r1 > 0)  _r = _r1;
        if(_r2 > 0)  _r = _r2;
    } else {
        _r = -c / b;
    }


    _y = b1 * _r + c1;
    _x = 0;

    //cout << _x << " " << _y << " = " << _r << endl;
}

const ld eps = 1e-12;

ld find_point(ld _x, ld _y)
{
    ld _r = sqrt((x[0] - _x) * (x[0] - _x) + (y[0] - _y) * (y[0] - _y)) - r[0];
    for(int i = 1; i < n; i ++)
    {
        _r = min(_r, sqrt((x[i] - _x) * (x[i] - _x) + (y[i] - _y) * (y[i] - _y)) - r[i]);
    }
    return _r;
}
bool check(ld _x, ld _y, ld _r)
{
    //cout << _x << " " << _y << " " << _r << endl;


    if(_x < 0 || _x > 1000 || _y < 0 || _y > 1000)
    {
            //cout << "bust boundary" << endl;
        return false;
    }
    if(isnan(_r) || _r > 1415 || isinf(_r))
    {
        return false;
    }

    for(int i = 0; i < n; i ++)
    {
        if((x[i] - _x) * (x[i] - _x) + (y[i] - _y) * (y[i] - _y) < (r[i] + _r) * (r[i] + _r) - eps)
        {
            //cout << "bust " << i << endl;
            return false;
        }
    }
    //cout << "ok\n";
    return true;
}


int main()
{
    ios::sync_with_stdio(false);
    cin.tie(0);


    cout << setprecision(13) << fixed;

    cin >> n;
    for(int i = 0; i < n; i ++)
    {
        cin >> x[i] >> y[i] >> r[i];
    }

    ld curr_best = 0;
    for(int i = 0; i < n; i ++)
    {
        for(int j = i + 1; j < n; j ++)
        {
            for(int k = j + 1; k < n; k ++)
            {
                int temp[] = {i, j, k};
                ld _x, _y, _r;
                fnd_tangent(temp, _x, _y, _r);
                if(_r > curr_best && check(_x, _y, _r))
                {
                    curr_best = _r;
                }
            }
        }
    }

    /// x = 0
    for(int i = 0; i < n; i ++)
    {
        for(int j = i + 1; j < n; j ++)
        {
            ld _x, _y, _r;
            fnd_online(x[i], y[i], r[i], x[j], y[j], r[j], _x, _y, _r);
            if(_r > curr_best && check(_x, _y, _r))
            {
                curr_best = _r;
            }
        }
    }

    /// x = 1000
    for(int i = 0; i < n; i ++)
    {
        for(int j = i + 1; j < n; j ++)
        {
            ld _x, _y, _r;
            fnd_online(x[i] - 1000, y[i], r[i], x[j] - 1000, y[j], r[j], _x, _y, _r);
            _x += 1000;
            if(_r > curr_best && check(_x, _y, _r))
            {
                curr_best = _r;
            }
        }
    }

    /// y = 0
    for(int i = 0; i < n; i ++)
    {
        for(int j = i + 1; j < n; j ++)
        {
            ld _x, _y, _r;
            fnd_online(y[i], x[i], r[i], y[j], x[j], r[j], _y, _x, _r);
            if(_r > curr_best && check(_x, _y, _r))
            {
                curr_best = _r;
            }
        }
    }

    /// y = 1000
    for(int i = 0; i < n; i ++)
    {
        for(int j = i + 1; j < n; j ++)
        {
            ld _x, _y, _r;
            fnd_online(y[i] - 1000, x[i], r[i], y[j] - 1000, x[j], r[j], _y, _x, _r);
            _y += 1000;
            if(_r > curr_best && check(_x, _y, _r))
            {
                curr_best = _r;
            }
        }
    }

    {
        ld _x = 0, _y = 0, _r = find_point(_x, _y);
        if(_r > curr_best && check(_x, _y, _r))
        {
            curr_best = _r;
        }
    }
    {
        ld _x = 0, _y = 1000, _r = find_point(_x, _y);
        if(_r > curr_best && check(_x, _y, _r))
        {
            curr_best = _r;
        }
    }
    {
        ld _x = 1000, _y = 0, _r = find_point(_x, _y);
        if(_r > curr_best && check(_x, _y, _r))
        {
            curr_best = _r;
        }
    }
    {
        ld _x = 1000, _y = 1000, _r = find_point(_x, _y);
        if(_r > curr_best && check(_x, _y, _r))
        {
            curr_best = _r;
        }
    }

    cout << curr_best;
}
/**
3
42.1337 13.666 191.17
987.987 666.666 321.321
133.331 666.666666666 123.321
*/


/**
7
42.1337 13.666 191.17
543.21 765.432 55.555
987.987 666.666 321.321
0 1000 100
666.666 66.66 16.6
133.331 666.666666666 123.321
888.8 88.8 8.8
*/
/**
2
20 20 100
500 500 30
*/
