#include <iostream>
#include <vector>
#include <cmath>
#include <tuple>
#include <iomanip>
using namespace std;

struct Circle {
	double x, y, r;
};
vector<Circle> circles;

int N;

constexpr double sqdist(const Circle &c, const double x, const double y) {
	return sqrt((c.x-x)*(c.x-x) + (c.y-y)*(c.y-y));
}

tuple<Circle, double, double> search(double minX, double minY, double maxX, double maxY) {
	if(minX < 0) minX = 0;
	if(minY < 0) minY = 0;
	if(maxX > 1000) maxX = 1000;
	if(maxY > 1000) maxY = 1000;

	Circle ans = {-1, -1, 0};
	double dx = (maxX - minX) / 50;
	double dy = (maxY - minY) / 50;
	if(dx < 1e-13) dx = 1e-13;
	if(dy < 1e-13) dy = 1e-13;
	for(double x = minX;x < maxX;x += dx) {
		for(double y = minY;y < maxY;y += dy) {
			double minD = 2000;
			for(auto &c : circles) {
				const double dist = sqdist(c, x, y) - c.r;
				minD = min(minD, dist);
			}
			if(minD > ans.r) {
				ans.r = minD;
				ans.x = x;
				ans.y = y;
			}
		}
	}
	return {ans, dx, dy};
}

int main() {
	cin >> N;
	circles.resize(N);
	for(int i = 0;i < N;i ++) {
		cin >> circles[i].x >> circles[i].y >> circles[i].r;
	}

	Circle ans = {500, 500, 0};
	double pdx = 500, pdy = 500;
	cout << std::fixed << std::setprecision(20);
	bool last = false;
	for(int iteration = 0;iteration < 50;iteration ++) {
		//cout << pdx << ' ' << pdy << endl;
		auto res = search(ans.x - 2*pdx, ans.y - 2*pdy, ans.x + 2*pdx, ans.y + 2*pdy);
		//cout << "Done " << iteration << endl;
		ans = get<0>(res);
		const double mul = (1.0 + double(iteration) / 100);
		pdx = get<1>(res) * mul;
		pdy = get<2>(res) * mul;
	}

	cout << std::fixed << std::setprecision(10) << ans.r << endl;

	return 0;
}
