#include <iostream>
#include <vector>
#include <cmath>
#include <tuple>
#include <iomanip>
using namespace std;

typedef long double LD;

struct Circle {
	LD x, y, r;
};
vector<Circle> circles;

int N;

constexpr LD sqdist(const Circle &c, const LD x, const LD y) {
	return sqrtl((c.x-x)*(c.x-x) + (c.y-y)*(c.y-y));
}

tuple<Circle, LD, LD> search(LD minX, LD minY, LD maxX, LD maxY) {
	if(minX < 0) minX = 0;
	if(minY < 0) minY = 0;
	if(maxX > 1000) maxX = 1000;
	if(maxY > 1000) maxY = 1000;

	Circle ans = {-1, -1, 0};
	LD dx = (maxX - minX) / 40;
	LD dy = (maxY - minY) / 40;
	if(dx < 1e-16) dx = 1e-16;
	if(dy < 1e-16) dy = 1e-16;
	for(LD x = minX;x < maxX;x += dx) {
		for(LD y = minY;y < maxY;y += dy) {
			LD minD = 2000;
			for(auto &c : circles) {
				const LD dist = sqdist(c, x, y) - c.r;
				minD = min(minD, dist);
			}
			if(minD > ans.r) {
				ans.r = minD;
				ans.x = x;
				ans.y = y;
			}
		}
	}
	return {ans, dx, dy};
}

int main() {
	cin >> N;
	circles.resize(N);
	for(int i = 0;i < N;i ++) {
		cin >> circles[i].x >> circles[i].y >> circles[i].r;
	}

	Circle ans = {500, 500, 0};
	LD pdx = 500, pdy = 500;
	cout << std::fixed << std::setprecision(20);
	bool last = false;
	for(int iteration = 0;iteration < 35;iteration ++) {
		//cout << pdx << ' ' << pdy << endl;
		auto res = search(ans.x - 2*pdx, ans.y - 2*pdy, ans.x + 2*pdx, ans.y + 2*pdy);
		//cout << "Done " << iteration << endl;
		ans = get<0>(res);
		const LD mul = 1.0;//(1.0 + LD(iteration) / 100);
		pdx = get<1>(res) * mul;
		pdy = get<2>(res) * mul;
	}

	cout << std::fixed << std::setprecision(14) << ans.r << endl;

	return 0;
}
