#include <iostream>
#include <vector>
#include <cmath>
#include <tuple>
#include <iomanip>
using namespace std;

typedef long double LD;

struct Circle {
	LD x, y, r;
};
vector<Circle> circles;

int N;

constexpr LD sqdist(const Circle &c, const LD x, const LD y) {
	return sqrtl((c.x-x)*(c.x-x) + (c.y-y)*(c.y-y));
}

const LD SC = 10;
bool debug = false;

tuple<Circle, LD, LD> search(LD minX, LD minY, LD maxX, LD maxY) {
	if(minX < 0) minX = 0;
	if(minY < 0) minY = 0;
	if(maxX > 1000*SC) maxX = 1000*SC;
	if(maxY > 1000*SC) maxY = 1000*SC;

	Circle ans = {-1, -1, 0};
	LD dx = (maxX - minX) / 500;
	LD dy = (maxY - minY) / 500;
	if(dx < 1e-15) dx = 1e-15;
	if(dy < 1e-15) dy = 1e-15;
	for(LD x = minX;x < maxX;x += dx) {
		for(LD y = minY;y < maxY;y += dy) {
			LD minD = 2000000;
			//if(debug) cerr << x << ' ' << y << endl;
			for(auto &c : circles) {
				const LD dist = sqdist(c, x, y) - c.r;
				minD = min(minD, dist);
			}
			if(minD > ans.r) {
				ans.r = minD;
				ans.x = x;
				ans.y = y;
			}
		}
	}
	return {ans, dx, dy};
}

int main() {
	cin >> N;
	circles.resize(N);
	for(int i = 0;i < N;i ++) {
		cin >> circles[i].x >> circles[i].y >> circles[i].r;
		circles[i].x *= SC;
		circles[i].y *= SC;
		circles[i].r *= SC;
	}

	Circle ans = {500*SC, 500*SC, 0};
	LD pdx = 500*SC, pdy = 500*SC;
	cout << std::fixed << std::setprecision(20);
	bool last = false;
	for(int iteration = 0;iteration < 50;iteration ++) {
		//cout << pdx << ' ' << pdy << endl;
		if(iteration == 16) debug = true;
		auto res = search(ans.x - 1.3*pdx, ans.y - 1.3*pdy, ans.x + 1.3*pdx, ans.y + 1.3*pdy);
		//cout << "Done " << iteration << endl;
		ans = get<0>(res);
		pdx = get<1>(res) * 100;
		pdy = get<2>(res) * 100;
	}

	//scaleback and search again
	for(int i = 0;i < N;i ++) {
		circles[i].x /= SC;
		circles[i].y /= SC;
		circles[i].r /= SC;
	}

	pdx /= SC;
	pdy /= SC;
	ans.x /= SC;
	ans.y /= SC;
	ans.r /= SC;

	LD SX = ans.x - 1e-8;
	LD SY = ans.y - 1e-8;
	LD EX = ans.x + 1e-8;
	LD EY = ans.y + 1e-8;
	if(SX < 0) SX = 0;
	if(SY < 0) SY = 0;
	if(EX > 1000) EX = 1000;
	if(EY > 1000) EY = 1000;
	for(LD x = SX;x < EX;x += 1e-10) {
		for(LD y = SY;y < EY;y += 1e-10) {
			LD minD = 2000;
			for(auto &c : circles) {
				const LD dist = sqdist(c, x, y) - c.r;
				minD = min(minD, dist);
			}
			if(minD > ans.r) {
				ans.r = minD;
				ans.x = x;
				ans.y = y;
			}
		}
	}
	cout << std::fixed << std::setprecision(12) << ans.r << endl;

	return 0;
}
