#include<iostream>
#include<vector>
#include<cmath>
#include<iomanip>

using namespace std;

const int MAXN = 105;
double eps = 0.000000001;

struct Point
{
    double x, y;

    Point(){}
    Point(double x, double y)
    {
        this->x = x;
        this->y = y;
    }

    bool isInside()
    {
        if(x>=0 && x<=1000 && y>=0 && y<=1000) return true;
        return false;
    }
};

double calcDistSquared(Point A, Point B)
{
    return (A.x-B.x)*(A.x-B.x) + (A.y-B.y)*(A.y-B.y);
}

double calcDist(Point A, Point B)
{
    return sqrt(calcDistSquared(A, B));
}


struct Circle
{
    Point O;
    double r;

    Circle(){}
    Circle(Point O, double r)
    {
        this->O = O;
        this->r = r;
    }
};

pair <double, double> solveQuadratic(double A, double B, double C)
{
    if(A==0) return {-C/B, -C/B};

    double D = B*B - 4*A*C;

    // << "D: " << fixed << setprecision(5) << D << '\n';

    if(D>-eps && D<0) D = 0;
    if(D<0) return {-1000, -1000};

    return {(-B-sqrt(D))/(2*A), (-B+sqrt(D))/(2*A)};
}

vector <Point> intersect(Circle A, Circle B)
{
    double x1 = A.O.x, y1 = A.O.y;
    double x2 = B.O.x - x1, y2 = B.O.y - y1;
    double r1 = A.r, r2 = B.r;

    if(x2!=0)
    {
        double c = (r1*r1-r2*r2+x2*x2+y2*y2);
        pair <double, double> s = solveQuadratic(4*x2*x2+4*y2*y2, -4*c*y2, c*c-4*x2*x2*r1*r1);

        Point P1 = Point(+sqrt(r1*r1-s.first*s.first)+x1, s.first+y1);
        Point P2 = Point(-sqrt(r1*r1-s.first*s.first)+x1, s.first+y1);

        Point P3 = Point(+sqrt(r1*r1-s.second*s.second)+x1, s.second+y1);
        Point P4 = Point(-sqrt(r1*r1-s.second*s.second)+x1, s.second+y1);

        //cout << P1.x << " " << P1.y << '\n';
        //cout << P2.x << " " << P2.y << '\n';
        //cout << P3.x << " " << P3.y << '\n';
        //cout << P4.x << " " << P4.y << '\n';

        return {P1, P2, P3, P4};
    }
    else
    {
        double c = (r1*r1-r2*r2+x2*x2+y2*y2);
        pair <double, double> s = solveQuadratic(4*x2*x2+4*y2*y2, -4*c*x2, c*c-4*y2*y2*r1*r1);

        Point P1 = Point(s.first+x1, +sqrt(r1*r1-s.first*s.first)+y1);
        Point P2 = Point(s.first+x1, -sqrt(r1*r1-s.first*s.first)+y1);

        Point P3 = Point(s.second+x1, +sqrt(r1*r1-s.second*s.second)+y1);
        Point P4 = Point(s.second+x1, -sqrt(r1*r1-s.second*s.second)+y1);

        return {P1, P2, P3, P4};
    }
}

int n;
Circle a[MAXN];

double getRadius(Point P)
{
    if(P.isInside()==false) return 0;

    double ans = 2000;
    for(int i = 0;i<n;i++)
    {
        ans = min(ans, calcDist(a[i].O, P)-a[i].r);
    }

    return ans;
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> n;
    for(int i = 0;i<n;i++)
    {
        cin >> a[i].O.x >> a[i].O.y >> a[i].r;
    }

    int stepLim = ((n<=20)?400:45);

    double answer = 0;
    for(int i = 0;i<n;i++)
    {
        for(int j = i+1;j<n;j++)
        {
            double l = (calcDist(a[i].O, a[j].O) - a[i].r - a[j].r)/2, r = 2000;

            //cout << l << " SDDAS " << r << " || " << calcDist(a[i].O, a[j].O) << '\n';
            for(int step = 0;step<stepLim;step++)
            {
                double mid = (l+r)/2;

                double maxDist = 0;
                vector <Point> v = intersect(Circle(a[i].O, a[i].r+mid), Circle(a[j].O, a[j].r+mid));
                for(Point p: v)
                {
                    maxDist = max(maxDist, getRadius(p));
                }

                if(maxDist+eps>=mid) l = mid;
                else r = mid;
            }

            double maxDist = 0;
            vector <Point> v = intersect(Circle(a[i].O, a[i].r+l), Circle(a[j].O, a[j].r+l));
            for(Point p: v)
            {
                maxDist = max(maxDist, getRadius(p));
            }
            answer = max(answer, maxDist);

        }
    }

    cout << fixed << setprecision(15) << answer << '\n';
}
