#include<iostream>
#include<vector>
#include<cstring>

using namespace std;

const int MAXN = 1555;

struct Edge
{
    int u, v, cap, ind;
};

struct MaxFlowGraph
{
    vector <Edge> edges;
    vector <int> graph[MAXN];

    int s = 1, t = 2;
    int ind[MAXN];
    bool used[MAXN];

    int addEdge(int u, int v)
    {
        //cout << "Edge: " << u << "->" << v << '\n';

        edges.push_back({u, v, 1, edges.size()});
        edges.push_back({v, u, 0, edges.size()});

        graph[u].push_back(edges.size()-2);
        graph[v].push_back(edges.size()-1);

        return edges.size() - 2;
    }

    int DFS(int x)
    {
        if(x==t) return 1;
        used[x] = true;

        //cout << x << '\n';

        for(int i = ind[x];i<graph[x].size();i++)
        {
            if(edges[ graph[x][i] ].cap>0 && used[ edges[ graph[x][i] ].v ]==false)
            {
                int flow = DFS(edges[ graph[x][i] ].v);
                if(flow>0)
                {
                    edges[ graph[x][i] ].cap -= flow;
                    edges[ graph[x][i]^1 ].cap += flow;

                    return flow;
                }
            }

            ind[x]++;
        }

        return 0;
    }

    int Arbeit(MaxFlowGraph &GOrig)
    {
        int maxFlow = 0;
        memset(ind, 0, sizeof(ind));

        while(true)
        {
            //cout << " --- " << '\n';
            memset(used, 0, sizeof(used));
            int flow = DFS(s);

            //cout << " -> " << flow << '\n';
            if(flow==0)
            {
                memset(ind, 0, sizeof(ind));
                memset(used, 0, sizeof(used));

                flow = DFS(s);

                if(flow==0) break;
            }

            maxFlow += flow;
        }

        edges = GOrig.edges;
        return maxFlow;
    }
};

int n, S, T;
pair <int, int> kids[MAXN];

MaxFlowGraph GOrig, G;
int song[MAXN], instr[MAXN];

int songCnt[MAXN], instrCnt[MAXN];
int kidsEdge[MAXN], songEdge[MAXN], instrEdge[MAXN];

void Init()
{
    int node = 3;
    for(int i = 1;i<=S;i++) song[i] = node, node++;
    for(int i = 1;i<=T;i++) instr[i] = node, node++;

    for(int x = 1;x<=S;x++) songEdge[x] = GOrig.addEdge(GOrig.s, song[x]);
    for(int x = 1;x<=T;x++) instrEdge[x] = GOrig.addEdge(instr[x], GOrig.t);
    for(int i = 1;i<=n;i++) kidsEdge[i] = GOrig.addEdge(song[ kids[i].first ], instr[ kids[i].second ]);

    G = GOrig;
}

void printVector(vector <int> &v)
{
    if(v.size()==0)
    {
        cout << '\n';
        return;
    }

    cout << v[0];
    for(int i = 1;i<v.size();i++) cout << " " << v[i];
    cout << '\n';
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    cin >> n >> S >> T;
    for(int i = 1;i<=n;i++)
    {
        int x, y;
        cin >> x >> y;

        songCnt[x]++;
        instrCnt[y]++;
        kids[i] = {x, y};
    }
    Init();

    int K = G.Arbeit(GOrig);

    vector <int> an, bn;
    for(int i = 1;i<=n;i++)
    {
        G.edges[ kidsEdge[i] ].cap = 0;
        if(G.Arbeit(GOrig)!=K) an.push_back(i);

        G.edges[ songEdge[ kids[i].first ] ].cap = 0;
        G.edges[ kidsEdge[i] ].cap = 0;
        G.edges[ instrEdge[ kids[i].second ] ].cap = 0;
        if(G.Arbeit(GOrig)<K-1)
        {
            songCnt[ kids[i].first ]--;
            instrCnt[ kids[i].second ]--;

            bn.push_back(i);
        }
    }

    vector <int> al, bl;
    for(int i = 1;i<=S;i++)
    {
        G.edges[ songEdge[i] ].cap = 0;


        //cout << i << " -> " << G.Arbeit(GOrig) << '\n';
        if(G.Arbeit(GOrig)!=K) al.push_back(i);

        if(songCnt[i]==0) bl.push_back(i);
    }

    vector <int> am, bm;
    for(int i = 1;i<=T;i++)
    {
        G.edges[ instrEdge[i] ].cap = 0;
        if(G.Arbeit(GOrig)!=K) am.push_back(i);

        if(instrCnt[i]==0) bm.push_back(i);
    }

    cout << K << " " << an.size() << " " << bn.size() << " " << al.size() << " " << bl.size() << " " << am.size() << " " << bm.size() << '\n';
    printVector(an);
    printVector(bn);
    printVector(al);
    printVector(bl);
    printVector(am);
    printVector(bm);
}
