#include<iostream>
#include<vector>
#include<cmath>
#include<iomanip>
#define eps 1e-11
using namespace std;
struct incen{
    double x,y,r;
};
struct line{
    double a,b,c;
};
vector<incen>incens;
pair<double,double>corners[4]={{0,0},{0,1000},{1000,1000},{1000,0}};
int n;
double msqrt(double a){
    double l=0,r=a;
    while(r-l>eps){
        double m=(l+r)/2.0;
        if(m*m>a){
            r=m;
        }else{
            l=m;
        }

    }
    return r;
}
bool intersect(incen a,incen b){
    double c=a.x-b.x,d=a.y-b.y;
    double dist=msqrt(c*c+d*d);
    //cout<<"check: "<<a.r<<" "<<b.r<<" "<<dist<<endl;
    if(a.r+b.r+eps<dist || a.r-eps>dist || b.r-eps>dist){
        //cout<<"fail: "<<a.r<<" "<<b.r<<" "<<dist<<endl;
        return 1;
    }
    return 0;
}
bool check(incen curr){
    //cout<<curr.r<<endl;
    for(int i=0;i<n;i++){
        if(intersect(curr,incens[i])){
            //cout<<"fail"<<endl;
            return 0;
        }
    }
    return 1;
}
line toline(double x1,double y1,double x2,double y2){
    double a=(y2-y1)/(x1+x2);
    double b=1;
    double c=a*x1+y1;
    return {a,b,c};
}
pair<double,double> intersect(line a,line b){
    double x=(a.c-b.c)/(a.a-b.a);
    double y=a.c-a.a*x;
    return {x,y};
}
pair<double,double> bs1(incen a,incen b){
    if(a.x>b.x)swap(a,b);
    double l=a.x,r=b.x;
    while(l+eps<r){
        double m=(l+r)/2.0;
        double d1=msqrt((a.x-m)*(a.x-m)+a.y*a.y)-a.r;
        double d2=msqrt((b.x-m)*(b.x-m)+b.y*b.y)-b.r;
        if(d1<d2){
            l=m;
        }else{
            r=m;
        }
    }
    double ans=msqrt((a.x-l)*(a.x-l)+a.y*a.y)-a.r;
    return {l,ans};
}
pair<double,double> bs2(incen a,incen b){
    if(a.x>b.x)swap(a,b);
    double l=a.x,r=b.x;
    while(l+eps<r){
        double m=(l+r)/2.0;
        double d1=msqrt((a.x-m)*(a.x-m)+(a.y-1000)*(a.y-1000))-a.r;
        double d2=msqrt((b.x-m)*(b.x-m)+(b.y-1000)*(b.y-1000))-b.r;
        if(d1<d2){
            l=m;
        }else{
            r=m;
        }
    }
    double ans=msqrt((a.x-l)*(a.x-l)+(a.y-1000)*(a.y-1000))-a.r;
    return {l,ans};
}
pair<double,double> bs3(incen a,incen b){
    if(a.y>b.y)swap(a,b);
    double l=a.y,r=b.y;
    while(l+eps<r){
        double m=(l+r)/2.0;
        double d1=msqrt(a.x*a.x+(a.y-m)*(a.y-m))-a.r;
        double d2=msqrt(b.x*b.x+(b.y-m)*(b.y-m))-b.r;
        if(d1<d2){
            l=m;
        }else{
            r=m;
        }
    }
    double ans=msqrt(a.x*a.x+(a.y-l)*(a.y-l))-a.r;
    return {l,ans};
}
pair<double,double> bs4(incen a,incen b){
    if(a.y>b.y)swap(a,b);
    double l=a.y,r=b.y;
    while(l+eps<r){
        double m=(l+r)/2.0;
        double d1=msqrt((a.x-1000)*(a.x-1000)+(a.y-m)*(a.y-m))-a.r;
        double d2=msqrt((b.x-1000)*(b.x-1000)+(b.y-m)*(b.y-m))-b.r;
        if(d1<d2){
            l=m;
        }else{
            r=m;
        }
    }
    double ans=msqrt((b.x-1000)*(b.x-1000)+(b.y-l)*(b.y-l))-b.r;
    return {l,ans};
}
int main(){
    //cout<<fixed<<setprecision(9)<<msqrt(10)<<endl;
    cin>>n;
    for(int i=0;i<n;i++){
        double a,b,c;
        cin>>a>>b>>c;
        incens.push_back({a,b,c});
    }
    double ans=0;
    for(int i=0;i<4;i++){
        for(int j=0;j<n;j++){
            double a=corners[i].first-incens[j].x,b=corners[i].second-incens[j].y;
            double cr=msqrt(a*a+b*b)-incens[j].r;
            //cout<<cr<<endl;
            if(cr<0)continue;
            if(check({corners[i].first,corners[i].second,cr})){
                ans=max(ans,cr);
            }
        }
    }
    for(int i=0;i<n;i++){
        for(int j=i+1;j<n;j++){
            pair<double,double> x1=bs1(incens[i],incens[j]);
            if(check({x1.first,0,x1.second})){
                ans=max(ans,x1.second);
            }
            pair<double,double> x2=bs2(incens[i],incens[j]);
            if(check({x2.first,1000,x2.second})){
                ans=max(ans,x2.second);
            }
            pair<double,double> y1=bs3(incens[i],incens[j]);
            if(check({0,y1.first,y1.second})){
                ans=max(ans,y1.second);
            }
            pair<double,double> y2=bs4(incens[i],incens[j]);
            if(check({1000,y2.first,y2.second})){
                ans=max(ans,y2.second);
            }
        }
    }
    cout<<fixed<<setprecision(11)<<ans<<endl;
    return 0;
}
